struct SPoint
{
  double mAsk;
  double mBid;
  double mLow;
  double mHigh;
  double mOpen;
  double mClose;
  double mPoint;
  double mSpread;
  double mVolume;
  datetime mTime;
};

struct SSymbol
{
  string mName;
  double mMean;
  double mUnit;
  double mOrder;
  double mAsk;
  double mBid;
};

struct SBet
{
  double mCount;
  double mMinus;
  double mPlus;
};

struct SSeries
{
  SSymbol mSymbol;
  SPoint mPoints[];
};

struct SSets
{
  SBet mBet;
  SSymbol mSymbol;
  double mAsk[];
  double mBid[];
  double mLow[];
  double mHigh[];
  double mOpen[];
  double mClose[];
  double mPoint[];
  double mPrice[];
  double mSpread[];
  long mVolume[];
  datetime mTime[];
};

struct SGroup
{
  string mName;
  string mMessage;
  double mPrice;
  double mMean;
};

struct SDeal
{
  int mEntry;
  int mDirection;
  ulong mTicket;
  string mName;
  double mPrice;
  double mVolume;
  double mSummary;
  datetime mTime;
};

struct SName
{
  string mData;
  SSets mSeries[];
};

class CHelpers
{
  protected : 
  
    datetime _openTime[];
      
  public : 

    /**
     * Check if string is null or empty
     */
    int isEmpty(const string data)
    {
      return data == NULL || data == "" || StringLen(data) == 0;
    }

    /**
     * Convert distance in pips to profit in money
     */
    double pipsToMoney(const string symbol, const double volume, const double pips)
    {
      double point = SymbolInfoDouble(symbol, SYMBOL_POINT);
      double step = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
      double value = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);
      return pips * point * value * volume / step;
    }

    /**
     * Convert profit in money to distance in pips
     */
    double moneyToPips(const string symbol, const double volume, const double amount)
    {
      double point = SymbolInfoDouble(symbol, SYMBOL_POINT);
      double step = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_SIZE);
      double value = SymbolInfoDouble(symbol, SYMBOL_TRADE_TICK_VALUE);

      if (value == 0 || volume == 0)
      {
        return 0;
      }

      return amount * step / value / volume;
    }

    /**
     * Convert profit in money to distance in pips
     */
    double getCommissionPips(const string symbol, const double volume, const double comission)
    {
      return moneyToPips(symbol, volume, getCommission(symbol, volume, comission));
    }

    /**
     * Correct commission value for selected symbol
     */
    double getCommission(const string symbol, const double volume, const double comission)
    {
      string base = SymbolInfoString(symbol, SYMBOL_CURRENCY_BASE);
      string quote = AccountInfoString(ACCOUNT_CURRENCY);
      double price = getCurrentPrice(base + quote);
      double value = comission * volume;

      if (base == AccountInfoString(ACCOUNT_CURRENCY)) 
      {
        return value;
      }

      if (StringFind(symbol, quote) >= 0) 
      {
        return value * getCurrentPrice(symbol);
      }

      return price ? value * price : value;
    }
    
    /**
     * Get minimal available lot for trading
     */
    int getVolumePrecision(const string symbol)
    {
      int digits = 0;
      double v = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MIN);
      for (int k = 1; k < 100 && MathFloor(v * k) == 0; k *= 10, digits++);
      return digits;
    }
    
    /**
     * Get number of minimum available bars in history
     */
    int getBars(const string symbol, const ENUM_TIMEFRAMES period, const int count = 0)
    {
      MqlRates rates[];

      int limit = count;
      int bars = Bars(symbol, period);
      
      if (bars > 0 && count > bars) 
      {
        limit = bars;
      }

      int items = CopyRates(symbol, period, 0, limit, rates);

      if (items < 1)
      {
        Print(
          "Synchronization : " + symbol + ", " + 
          "Bars : " + IntegerToString(items));

        return 0;
      }
      
      return items;
    }

    /**
     * Get margin needed to open new position
     */
    double getMargin(const string symbol, const double volume)
    {
      long leverage = AccountInfoInteger(ACCOUNT_LEVERAGE);
      double contract = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
      double price = getCurrencyCost(symbol);
      return price * volume * contract / leverage;
    }

    /**
     * Check if it's a good time to open new position
     */
    bool isTradingTime(const string shours = "0:30-23:30", const string sdays = "0-6")
    {
      string days[];
      string hours[];
      
      StringSplit(sdays, '-', days);
      StringSplit(shours, '-', hours);
  
      MqlDateTime o = {0};
      datetime now = TimeCurrent(o);
      
      string day = 
        string(o.year) + "." + 
        string(o.mon) + "." + 
        string(o.day);
  
      datetime H1 = StringToTime(day + " " + hours[0]);
      
      if (hours[0] > hours[1])
      {
        day = 
          string(o.year) + "." + 
          string(o.mon) + "." + 
          string(o.day + 1);
      }
      
      datetime H2 = StringToTime(day + " " + hours[1]);
      
      int D1 = int(days[0]);
      int D2 = int(days[ArraySize(hours) - 1]);
  
      if (now >= H1 && now <= H2 && o.day_of_week >= D1 && o.day_of_week <= D2)
      { 
        return true;
      }
  
      return false;
    }

    /**
     * Get cost of minimal price move measured in account's currency
     */
    double getTickCost(const string symbol, const double volume = 1)
    {
      string parts[];

      int count = StringSplit(symbol, '.', parts);
      
      double point = SymbolInfoDouble(symbol, SYMBOL_POINT);
      double contract = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
      
      string currency = AccountInfoString(ACCOUNT_CURRENCY);
      string base = SymbolInfoString(symbol, SYMBOL_CURRENCY_BASE);
      string quote = SymbolInfoString(symbol, SYMBOL_CURRENCY_PROFIT);

      if (quote == currency)
      {
        return contract * volume;
      }

      if (base == currency)
      {
        return contract * volume / getCurrentPrice(symbol);
      }

      string source1 = base + currency;
      string source2 = currency + base;
      string reverse1 = quote + currency;
      string reverse2 = currency + quote;

      if (count > 1)
      {
        source1 += "." + parts[1];
        source2 += "." + parts[1];
        reverse1 += "." + parts[1];
        reverse2 += "." + parts[1];
      }

      if (SymbolSelect(reverse1, true))
      {
        return contract * volume * getCurrentPrice(reverse1);
      }
      
      return contract * volume / getCurrentPrice(reverse2);
    }

    /**
     * Get date and time by number of bar starting from the most recent
     */
    datetime positionToTime(const int position, const ENUM_TIMEFRAMES period = 0)
    {
      int day = PeriodSeconds(PERIOD_D1);
      int seconds = PeriodSeconds(period);
      datetime date = getLastServerTime();

      date -= date % MathMin(day, seconds);

      for (int k = 0; k < position; k++)
      {
        date -= seconds;
      }

      return date;
    }

    /**
     * Get time of the last tick for specified symbol
     */
    datetime getLastServerTime()
    {
      datetime moment = TimeCurrent();
      datetime alternative = TimeTradeServer();

      if (alternative > moment) 
      {
        moment = alternative;
      }

      return moment;
    }

    /**
     * Get time of the last tick for specified symbol
     */
    datetime getLastSymbolTime(const string symbol)
    {
      return (datetime) SymbolInfoInteger(symbol, SYMBOL_TIME);
    }

    /**
     * Basic smoothing algorithm
     */
    double EMA(const double current, const double previous, const int period)
    {
      double result = 0.0;
  
      if (period > 0)
      {
        double pr = 2.0 / (period + 1.0);
        result = current * pr + previous * (1 - pr);
      }
  
      return result;
    }

    /**
     * Get list of deals in history based on parameters
     */
    int getDeals(
        SDeal &deals[], 
        const string symbol = NULL, 
        const ENUM_DEAL_ENTRY inout = NULL, 
        const ENUM_DEAL_TYPE direction = NULL)
    {
      ArrayResize(deals, 0);
  
      if (HistorySelect(0, TimeCurrent()))
      {
        int index = 0;
        int count = HistoryDealsTotal();
        
        for (int k = 0; k < count; k++)
        {
          ulong ticket = HistoryDealGetTicket(k);
          ENUM_DEAL_TYPE route = (ENUM_DEAL_TYPE) HistoryDealGetInteger(ticket, DEAL_TYPE);
          ENUM_DEAL_ENTRY deal = (ENUM_DEAL_ENTRY) HistoryDealGetInteger(ticket, DEAL_ENTRY);

          bool A = inout == NULL || inout == deal;
          bool B = direction == NULL || direction == route;

          if (A && B)
          {
            ArrayResize(deals, index + 1);
            deals[index].mEntry = deal;
            deals[index].mTicket = ticket;
            deals[index].mDirection = route == DEAL_TYPE_BUY ? 1 : -1;
            deals[index].mName = HistoryDealGetString(ticket, DEAL_SYMBOL);
            deals[index].mPrice = HistoryDealGetDouble(ticket, DEAL_PRICE);
            deals[index].mSummary = HistoryDealGetDouble(ticket, DEAL_PROFIT);
            deals[index].mTime = datetime(HistoryDealGetInteger(ticket, DEAL_TIME));
            deals[index].mVolume = HistoryDealGetDouble(ticket, DEAL_VOLUME);
            index++;
          }
        }
      }
      
      return ArraySize(deals);
    }
    
    /**
     * Output specified string to log file
     */
    void debug(const string data, const bool create = true)
    {
      int FH = FileOpen("debug.log", FILE_WRITE | FILE_READ | FILE_SHARE_READ | FILE_ANSI);
      
      FileSeek(FH, 0, create ? SEEK_SET : SEEK_END);
      
      FileWrite(FH, data);
      FileFlush(FH);
      FileClose(FH);
    }

    /**
     * Create sequence of bits represented as array to iterate over it
     */
    int getChain(int &codes[], const int order)
    {
      ArraySetAsSeries(codes, true);

      for (int k = 0; k < order; k++) 
      {
        if (codes[k] == 0)
        {
          codes[k] = 1;
          return 1;
        }
        else
        {
          codes[k] = 0;
        }
      }
  
      return 0;
    }
    
    /**
     * Check if new bar has come
     */
    bool isNewPeriod(const string symbol, const int index = 0, const int bars = 1, const ENUM_TIMEFRAMES period = PERIOD_CURRENT)
    {
      datetime open = (datetime) SeriesInfoInteger(symbol, period, SERIES_LASTBAR_DATE);

      if (ArraySize(_openTime) < index + 1)
      {
        ArrayResize(_openTime, index + 1);
      }

      int range = int(open) - int(_openTime[index]);

      if (range >= PeriodSeconds(period) * bars)
      {
        _openTime[index] = open;
        return true;
      }
      
      return false;
    }
    
    /**
     * Accumulate messages and print them as a comment on the chart
     */
    string setMessage(const string message)
    {
      static string content;
  
      if (StringLen(message) > 0)
      {
        content += message + "\n";
        Comment(content);
      }
      
      return content;
    }
    
    /**
     * Get random number, may be restricted to binary only
     */
    int getRandom(const bool binary, const int size = 0)
    {
      int limit = size;

      if (limit <= 0 || limit >= 32768)
      {
        limit = 32768;
      }
  
      int number = 1 + limit * MathRand() / 32768;
  
      if (binary)
      {
        return number % 2 == 0 ? 1 : -1;
      }
  
      return number;
    }

    /**
     * Get precision for the symbol
     */
    int getPrecision(const string symbol)
    {
      return (int) SymbolInfoInteger(symbol, SYMBOL_DIGITS);
    }

    /**
     * Get tick size
     */
    double getPrecisionValue(const string symbol)
    {
      return SymbolInfoDouble(symbol, SYMBOL_POINT);
    }

    /**
     * Get last bar data
     */
    MqlRates getBarData(const string symbol, ENUM_TIMEFRAMES period, int position = 0)
    {
      MqlRates rate = {0};
      MqlRates rates[];

      int precision = getPrecision(symbol);

      if (CopyRates(symbol, period, position, 1, rates))
      {
        return rates[0];
      }

      return rate;
    }

    /**
     * Get price for specified operation
     */
    double getCurrentPrice(const string symbol, const int operation = 0, const double price = 0)
    {
      if (price) 
      {
        return price;
      }

      int precision = getPrecision(symbol);
      double ask = SymbolInfoDouble(symbol, SYMBOL_ASK);
      double bid = SymbolInfoDouble(symbol, SYMBOL_BID);

      if (operation > 0) 
      {
        return NormalizeDouble(ask, precision);
      }

      if (operation < 0) 
      {
        return NormalizeDouble(bid, precision);
      }

      return NormalizeDouble((ask + bid) / 2, precision);
    }

    /**
     * Get price of the asset in the currency of the deposit
     */
    double getCurrencyCost(const string symbol, double price = 0)
    {
      string currency = AccountInfoString(ACCOUNT_CURRENCY);
      string base = SymbolInfoString(symbol, SYMBOL_CURRENCY_BASE);
      string quote = SymbolInfoString(symbol, SYMBOL_CURRENCY_PROFIT);

      price = price ? price : getCurrentPrice(symbol);

      if (currency == quote)
      {
        return price;
      }

      if (currency == base)
      {
        return 1 / price;
      }

      string parts[];
      string source = base + currency;
      string reverse = currency + base;

      int count = StringSplit(symbol, '.', parts);

      if (count > 1)
      {
        source += "." + parts[1];
        reverse += "." + parts[1];
      }

      if (SymbolSelect(source, true))
      {
        return getCurrentPrice(source);
      }

      if (SymbolSelect(reverse, true))
      {
        return 1 / getCurrentPrice(reverse);
      }

      return price;
    }

    /**
     * Get biggest lot size that can be used according to current account balance 
     */
    double getAvailableVolume(const string symbol)
    {
      long leverage = AccountInfoInteger(ACCOUNT_LEVERAGE);
      double balance = AccountInfoDouble(ACCOUNT_MARGIN_FREE);
      double contract = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
      double price = getCurrencyCost(symbol);
      return NormalizeDouble(balance * leverage / contract / price, 2);
    }

    /**
     * Calculate position size according to selected risk / reward ratio
     */
    double getSize(const string symbol, const double percentage)
    {
      int precision = getVolumePrecision(symbol);
      int leverage = (int) AccountInfoInteger(ACCOUNT_LEVERAGE);
      double balance = AccountInfoDouble(ACCOUNT_BALANCE);
      double min = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MIN);
      double max = SymbolInfoDouble(symbol, SYMBOL_VOLUME_MAX);
      double contract = SymbolInfoDouble(symbol, SYMBOL_TRADE_CONTRACT_SIZE);
      double volume = balance * percentage * leverage / 100 / contract;

      return NormalizeDouble(MathMax(MathMin(volume, max), min), precision);
    }
    
    /**
     * Calculate new average for known sequence
     */
    double addToAverage(double average, int size, double value)
    {
      return (size * average + value) / (size + 1);
    }
};